/**
 * @param {Array<string>} argv
 * @param {string} long
 * @param {string} [short]
 * @param {boolean} [bool]
 * @param {boolean} [number]
 */
const find = (argv, long, short, bool = false, number = false) => {
  const re = short ? new RegExp(`^-(${short}|-${long})$`)
    : new RegExp(`^--${long}$`)
  const i = argv.findIndex(a => re.test(a))
  if (i == -1) return { argv }

  if (bool) {
    return {
      value: true,
      argv: [
        ...argv.slice(0, i),
        ...argv.slice(i + 1),
      ],
    }
  }

  const j = i + 1
  let value = argv[j]

  if (!value || (typeof value == 'string' && value.startsWith('--'))) return { argv }

  if (number) {
    value = parseInt(value, 10)
  }
  return {
    value,
    argv: [
      ...argv.slice(0, i),
      ...argv.slice(j + 1),
    ],
  }
}

/**
 * Parse the config and extract arguments from the `process.argv` array.
 * @param {!_argufy.Config} config The configuration for parsing, where each key is a flag name and values are either strings, or objects with possible properties:
 *
 * - __short__ Shorthand for this argument, usually one letter.
 * - __boolean__ Does not have to be followed by a value, true if given.
 * - __number__ Parse as a number.
 * - __command__ Whether this argument can be specified as the first argument without the dash, such as `example run`.
 * - __multiple__ Extract multiple commands as an  array when `command` is set to true.
 *
 * @example
```
const config = {
  title: 't',       // program -t Title
  open: {           // program -o
    short: 'o',
    boolean: true,
  },
  delay: {          // program -d 100
    short: 'd',
    number: true,
  },
  file: {           // program File.txt
    command: true,
  },
  file2: {           // program File.txt File2.txt
    command: true,
    multiple: true,
  },
}
```
 * @param {!Array<string>} [args] Array with arguments to parse. `process.argv` is used by default. It is assumed that user arguments start from the 3rd position.
 * @returns {Object<string, string|Array<string>|boolean|number>} An object with all found values for the configuration request.
 */
export default function argufy(config = {}, args = process.argv) {
  let [, , ...argv] = args
  const titles = findTitles(argv)
  argv = argv.slice(titles.length)
  let commandFound = !titles.length
  const res = Object.keys(config).reduce(({ _argv, ...acc }, key) => {
    if (_argv.length == 0 && commandFound) return { _argv, ...acc }
    const val = config[key]
    let value
    if (typeof val == 'string') {
      ({ value, argv: _argv } = find(_argv, key, val))
    } else {
      try {
        const { short, boolean, number, command, multiple } = val
        if (command && multiple && titles.length) {
          value = titles
          commandFound = true
        } else if (command && titles.length) {
          value = titles[0]
          commandFound = true
        } else {
          ({ value, argv: _argv } = find(_argv, key, short, boolean, number))
        }
      } catch (err) {
        return { _argv, ...acc }
      }
    }
    if (value === undefined) return { _argv, ...acc }
    const r = { _argv, ...acc, [key]: value }
    return r
  }, {
    _argv: argv,
  })
  return res
}

/** @param {!Array<string>} argv */
const findTitles = argv => {
  const titles = []
  for (let i = 0; i < argv.length; i++) {
    const a = argv[i]
    if (a.startsWith('-')) break
    titles.push(a)
  }
  return titles
}

/**
 * Based on the argufy config, creates an object that can be passed to `usually`.
 * @param {!_argufy.Config} argsConfig The configuration for parsing, where each key is a flag name and values are either strings, or objects with possible properties:
 * @example
```
{ 'input': 'The input to the program',
  '--output, -o': 'The output of the program', }
```
 */
export const reduceUsage = (argsConfig) => {
  const a = Object.keys(argsConfig).reduce((acc, key) => {
    const val = argsConfig[key]
    if (typeof val == 'string') {
      const k = `-${val}`
      acc[k] = ''
      return acc
    }
    let k = val.command ? key : `--${key}`
    if (val.short) k = `${k}, -${val.short}`
    let d = val.description
    if (val.default) d = `${d}\nDefault: ${val.default}.`
    acc[k] = d
    return acc
  }, {})
  return a
}

/* documentary types/index.xml */
/**
 * @suppress {nonStandardJsDocs}
 * @typedef {_argufy.Config} Config The configuration for parsing, where each key is a flag name and values are either strings, or objects with possible properties:
 */
/**
 * @suppress {nonStandardJsDocs}
 * @typedef {Object<string, string|!_argufy.Flag>} _argufy.Config The configuration for parsing, where each key is a flag name and values are either strings, or objects with possible properties:
 */
/**
 * @suppress {nonStandardJsDocs}
 * @typedef {_argufy.Flag} Flag The flag passed to the program.
 */
/**
 * @suppress {nonStandardJsDocs}
 * @typedef {Object} _argufy.Flag The flag passed to the program.
 * @prop {string} [short] Shorthand for this argument, usually one letter.
 * @prop {boolean} [boolean=false] Whether the flag is a boolean and does not require a value. Default `false`.
 * @prop {boolean} [number=false] Specifies whether the flag should be parsed as a number. Default `false`.
 * @prop {boolean} [command=false] If set to true, the value is read from the first argument passed to the CLI command (e.g., `$ cli command`). Default `false`.
 * @prop {boolean} [multiple=false] When using the `command` property, will parse the commands as an array. Default `false`.
 * @prop {string} [default] The default value for the argument. Does not actually set the value, only used in reducing the usage info (`argufy` bin on the other hand will set the default).
 * @prop {string} [description] The description to be used by `usually`.
 */
